
import numpy as np
from scipy import stats
from astropy.table import Table
from matplotlib import pyplot as plt
from scipy import interpolate as intp

def apl(ax, n, det='HLV', lab='HLV', src='bns', col='blue', sty='-', wid=3):
    dir = 'run/O3_'+det+'/'+src+'_astro/'
    allsky = Table.read(dir+'allsky.dat', format='ascii.fast_tab')
    coincs = Table.read(dir+'coincs.dat', format='ascii.fast_tab')

    area = allsky['area(90)']
    ifos = coincs['ifos']

    data = []
    for i in range(0, len(area)):
        if det == 'HLV' and n == 2:
            if ifos[i] == 'H1,L1' or ifos[i] == 'H1,V1' or ifos[i] == 'L1,V1':
                data.append(area[i])
        if det == 'HLV' and n == 3:
            if ifos[i] == 'H1,L1,V1':
                data.append(area[i])
        if det == 'HLVK' and n == 3:
            if ifos[i] == 'H1,K1,L1' or ifos[i] == 'H1,K1,V1' or\
                                        ifos[i] == 'K1,L1,V1':
                data.append(area[i])
        if det == 'HLVK' and n == 4:
            if ifos[i] == 'H1,K1,L1,V1':
                data.append(area[i])

    t = np.geomspace(ax.get_xlim()[0], ax.get_xlim()[1], 100)
    kde = stats.gaussian_kde(np.asarray(np.log(data)))
    (std,), = np.sqrt(kde.covariance)
    y = stats.norm(kde.dataset.ravel(), std).\
        cdf(np.log(t)[:, np.newaxis]).mean(1)

    ax.semilogx(t, y, color = col, linestyle = sty, linewidth = wid,\
                label = 'N='+str(n)+' '+format(lab, '75s'))

    ff  = intp.interp1d(y, t)
    return ff(0.5)


def plot(src, fsiz=15):
    fig = plt.figure(dpi=80, facecolor='white')
    ax  = fig.add_subplot(1, 1, 1)

    ax.set_xlim(1e-1, 1e4)
    ax.set_ylim(0, 1)
    ax.set_yticks([0, 0.25, 0.50, 0.75, 1])
    ax.set_xlabel(r'90% credible area (deg$^2$)',  fontsize=fsiz)
    ax.set_ylabel('Cumulative fraction of events', fontsize=fsiz)

    plt.tick_params(labelsize = fsiz)

    ax.axhline(0.5, color='black', linestyle=':')

    a1= apl(ax, 2, 'HLV',  '(HL,HV,LV)',    src=src, col='#0073b2', sty='--')
    a2= apl(ax, 3, 'HLVK', '(HLK,HVK,LVK)', src=src, col='#de8f07', sty='--')
    a3= apl(ax, 3, 'HLV',  '(HLV)',         src=src, col='#0073b2', sty='-')
    a4= apl(ax, 4, 'HLVK', '(HLVK)',        src=src, col='#de8f07', sty='-')

    aref = [ 16, 39 ]   # GW170817 and GW170818

    if src == 'bns':
        ax.axvline(aref[0], color='black', linestyle='--')
        ax.text(aref[0]*0.9, 0.7, 'GW170817', ha='right')

    if src == 'bbh':
        ax.axvline(aref[1], color='black', linestyle='--')
        ax.text(aref[1]*0.9, 0.7, 'GW170818', ha='right')

    ax.legend(fontsize=fsiz-4, loc='upper left')

    aa = [ a1, a2, a3, a4 ]
    for i in range(0, 4):
        x = 0.40
        y = 0.855-i*0.037
        fmt = '4.1f' if aa[i] < 100 else '3.0f'
        fig.text(x, y, 'median= ' +format(aa[i], fmt)+' deg$^2$')

    src = src.upper()
    fig.text(0.51, 0.99, 'O3 '+src, ha='center', va='top', size=fsiz+5)

    return fig

for src in [ 'bns', 'bbh', 'nsbh' ]:
    fig = plot(src)
    fig.show()
    fig.savefig('area_O3_'+src+'.pdf')
